@@ -32,7 +32,7 @@ Create a service:
   sub on_startup {
       my $self = shift;
       
-      $self->accept_jobs( 'my.service.echo' => 'echo' );
+      $self->accept_remote_calls( 'my.service.echo' => 'echo' );
       
       $self->accept_notifications( 'my.service.msg' => 'msg' );
   }
@@ -67,7 +67,7 @@ Create an API for the service:
       my ($class, %args) = @_;
       my $cli = Beekeeper::Client->instance;
       
-      my $result = $cli->do_job(
+      my $result = $cli->call_remote(
           method => "my.service.echo",
           params => { %args },
       );
@@ -170,7 +170,7 @@ A worker class just declares on startup which methods it will accept, then imple
   sub on_startup {
       my $self = shift;
   
-      $self->accept_jobs(
+      $self->accept_remote_calls(
           'myapp.str.uc' => 'uppercase',
       );
   }
@@ -199,7 +199,7 @@ This is the interface of the above service:
   
       my $client = Beekeeper::Client->instance;
   
-      my $resp = $client->do_job(
+      my $resp = $client->call_remote(
           method => 'myapp.str.uc',
           params => { string => $str },
       );
@@ -225,8 +225,8 @@ The file C<pool.config.json> defines all worker pools running on a host, specify
 bus should be used and which services it will run. For example:
 
   [{
-      "pool-id" : "myapp",
-      "bus-id"  : "backend",
+      "pool_id" : "myapp",
+      "bus_id"  : "backend",
       "workers" : {
           "MyApp::Worker" : { "workers_count" : 4 },
       },
@@ -236,7 +236,7 @@ The file C<bus.config.json> defines all logical buses used by the application, s
 connection parameters to the brokers that will service them. For example:
 
   [{
-      "bus-id"   : "backend",
+      "bus_id"   : "backend",
       "host"     : "localhost",
       "username" : "backend",
       "password" : "def456",
@@ -252,7 +252,7 @@ in these two files.
 To start or stop a pool of workers you use the C<bkpr> command. Given the above example config, 
 this will start 4 processes running C<MyApp::Worker> code:
 
-  bkpr --pool-id "myapp" start
+  bkpr --pool "myapp" start
 
 When started it daemonizes itself and forks all worker processes, then continues monitoring 
 those forked processes and immediately respawns defunct ones.
@@ -272,13 +272,13 @@ Beekeeper is pretty lightweight, so the performance depends mostly on *the broke
 These are 
 ballpark performance measurements of a local setup running Mosquitto:
 
-- A C<do_job> synchronous call to a remote method adds 1.5 ms of latency and involves 4 network 
+- A C<call_remote> synchronous call to a remote method adds 1.5 ms of latency and involves 4 network 
 round trips. This implies a maximum of 650 synchronous calls per second.
 
-- A C<do_async_job> asynchronous call to a remote method takes 0.1 ms. This implies a maximum 
+- A C<call_remote_async> asynchronous call to a remote method takes 0.1 ms. This implies a maximum 
 of 10000 asynchronous calls per second (just the call, then it must wait for responses).
 
-- Scheduling a remote task with C<do_background_job> takes 0.1 ms. This implies a maximum of 
+- Scheduling a remote task with C<fire_remote> takes 0.1 ms. This implies a maximum of 
 10000 calls per second.
 
 - Sending a notification with C<send_notification> takes 0.1 ms. A worker can emit 10000 notifications 
