@@ -12,6 +12,7 @@ use Beekeeper::Config;
 use JSON::XS;
 use Sys::Hostname;
 use Time::HiRes;
+use Digest::SHA 'sha256_hex';
 use Carp;
 
 use constant QUEUE_LANES => 2;
@@ -21,14 +22,16 @@ use Exporter 'import';
 
 our @EXPORT_OK = qw(
     send_notification
-    do_job
-    do_async_job
-    do_background_job
-    wait_all_jobs
-    set_auth_tokens
-    get_auth_tokens
+    call_remote
+    call_remote_async
+    fire_remote
+    wait_async_calls
+    get_authentication_data
+    set_authentication_data
+
     __do_rpc_request
     __create_response_topic
+    __use_authorization_token
 );
 
 our %EXPORT_TAGS = ('worker' => \@EXPORT_OK );
@@ -49,8 +52,9 @@ sub new {
         response_topic => undef,
         in_progress    => undef,
         curr_request   => undef,
-        auth_tokens    => undef,
-        session_id     => undef,
+        caller_id      => undef,
+        caller_addr    => undef,
+        auth_data      => undef,
         async_cv       => undef,
         correlation_id => 1,
         callbacks      => {},
@@ -79,7 +83,7 @@ sub new {
                 # Use default parameters (if any)
                 my ($default) = grep { $config->{$_}->{default} } keys %$config;
                 croak "No default bus defined into config file bus.config.json" unless $default;
-                $bus_id = $config->{$default}->{'bus-id'};
+                $bus_id = $config->{$default}->{'bus_id'};
                 %args = ( %{$config->{$default}}, bus_id => $bus_id, %args );
             }
         }
@@ -146,7 +150,7 @@ sub send_notification {
 
     my %send_args;
 
-    my $local_bus = $self->{_BUS}->{cluster};
+    my $local_bus = $self->{_BUS}->{bus_role};
 
     $remote_bus = $self->{_CLIENT}->{forward_to} unless (defined $remote_bus);
 
@@ -164,13 +168,8 @@ sub send_notification {
         $send_args{'topic'} =~ tr|.|/|;
     }
 
-    if (exists $args{'__auth'}) {
-        $send_args{'x-auth-tokens'} = $args{'__auth'};
-    }
-    else {
-        $send_args{'x-auth-tokens'} = $self->{_CLIENT}->{auth_tokens}  if defined $self->{_CLIENT}->{auth_tokens};
-        $send_args{'x-session'}     = $self->{_CLIENT}->{session_id}   if defined $self->{_CLIENT}->{session_id};
-    }
+    $send_args{'auth'} = $self->{_CLIENT}->{auth_data} if defined $self->{_CLIENT}->{auth_data};
+    $send_args{'clid'} = $self->{_CLIENT}->{caller_id} if defined $self->{_CLIENT}->{caller_id};
 
     if (exists $args{'buffer_id'}) {
         $send_args{'buffer_id'} = $args{'buffer_id'};
@@ -203,7 +202,7 @@ sub accept_notifications {
 
         #TODO: Allow to accept private notifications without subscribing
 
-        my $local_bus = $self->{_BUS}->{cluster};
+        my $local_bus = $self->{_BUS}->{bus_role};
 
         my $topic = "msg/$local_bus/$service/$method";
         $topic =~ tr|.*|/#|;
@@ -267,7 +266,7 @@ sub stop_accepting_notifications {
             next;
         }
 
-        my $local_bus = $self->{_BUS}->{cluster};
+        my $local_bus = $self->{_BUS}->{bus_role};
 
         my $topic = "msg/$local_bus/$service/$method";
         $topic =~ tr|.*|/#|;
@@ -297,7 +296,7 @@ sub stop_accepting_notifications {
 
 our $AE_WAITING;
 
-sub do_job {
+sub call_remote {
     my $self = shift;
 
     my $req = $self->__do_rpc_request( @_, req_type => 'SYNCHRONOUS' );
@@ -321,7 +320,7 @@ sub do_job {
     return $resp;
 }
 
-sub do_async_job {
+sub call_remote_async {
     my $self = shift;
 
     my $req = $self->__do_rpc_request( @_, req_type => 'ASYNCHRONOUS' );
@@ -329,7 +328,7 @@ sub do_async_job {
     return $req;
 }
 
-sub do_background_job {
+sub fire_remote {
     my $self = shift;
 
     # Send job to a worker, but do not wait for result
@@ -356,7 +355,7 @@ sub __do_rpc_request {
 
     my %send_args;
 
-    my $local_bus = $self->{_BUS}->{cluster};
+    my $local_bus = $self->{_BUS}->{bus_role};
 
     $remote_bus = $client->{forward_to} unless (defined $remote_bus);
 
@@ -377,17 +376,8 @@ sub __do_rpc_request {
         $send_args{'topic'} =~ tr|.|/|;
     }
 
-    if (exists $args{'__auth'}) {
-        $send_args{'x-auth-tokens'} = $args{'__auth'};
-    }
-    else {
-        $send_args{'x-auth-tokens'} = $client->{auth_tokens}  if defined $client->{auth_tokens};
-        $send_args{'x-session'}     = $client->{session_id}   if defined $client->{session_id};
-    }
-
-    my $timeout = $args{'timeout'} || REQ_TIMEOUT;
-    $send_args{'message_expiry'} = $timeout;
-
+    $send_args{'auth'} = $client->{auth_data} if defined $client->{auth_data};
+    $send_args{'clid'} = $client->{caller_id} if defined $client->{caller_id};
 
     my $BACKGROUND  = $args{req_type} eq 'BACKGROUND';
     my $SYNCHRONOUS = $args{req_type} eq 'SYNCHRONOUS';
@@ -463,6 +453,7 @@ sub __do_rpc_request {
     if ($__now != time) { $__now = time; AnyEvent->now_update }
 
     # Request timeout timer
+    my $timeout = $args{'timeout'} || REQ_TIMEOUT;
     $req->{_timeout} = AnyEvent->timer( after => $timeout, cb => sub {
         my $req = delete $client->{in_progress}->{$req_id};
         $req->{_response} = Beekeeper::JSONRPC::Error->request_timeout;
@@ -557,10 +548,10 @@ sub __create_response_topic {
     return $response_topic;
 }
 
-sub wait_all_jobs {
+sub wait_async_calls {
     my ($self) = @_;
 
-    # Wait for all pending jobs
+    # Wait for all pending async requests
     my $cv = delete $self->{_CLIENT}->{async_cv};
     return unless defined $cv;
 
@@ -574,20 +565,45 @@ sub wait_all_jobs {
 }
 
 
-sub set_auth_tokens {
-    my ($self, @tokens) = @_;
+sub get_authentication_data {
+    my ($self) = @_;
 
-    foreach my $token (@tokens) {
-        croak "Invalid token $token" unless (defined $token && length $token && $token !~ m/[\x00\n\|]/);
-    }
-
-    $self->{_CLIENT}->{auth_tokens} = join('|', @tokens);
+    $self->{_CLIENT}->{auth_data};
 }
 
-sub get_auth_tokens {
-    my $self = shift;
+sub set_authentication_data {
+    my ($self, $data) = @_;
 
-    return split(/\|/, $self->{_CLIENT}->{auth_tokens});
+    $self->{_CLIENT}->{auth_data} = $data;
+}
+
+sub __use_authorization_token {
+    my ($self, $token) = @_;
+
+    my $secret = 'salt'; #TODO: read from config file
+
+    my $adata_ref = \$self->{_CLIENT}->{auth_data};
+
+    my $guard = Beekeeper::Client::Guard->new( $adata_ref );
+
+    $$adata_ref = sha256_hex($token . $secret);
+
+    return $guard;
+}
+
+1;
+
+package Beekeeper::Client::Guard;
+
+sub new {
+    my ($class, $ref) = @_;
+
+    bless [$ref, $$ref], $class;
+}
+
+sub DESTROY {
+
+    ${$_[0]->[0]} = $_[0]->[1];
 }
 
 1;
@@ -615,7 +631,7 @@ Version 0.03
       params => { foo => $foo },
   );
   
-  my $resp = $client->do_job(
+  my $resp = $client->call_remote(
       method => "my.service.bar",
       params => { %args },
   );
@@ -624,7 +640,7 @@ Version 0.03
   
   print $resp->result;
   
-  my $req = $client->do_async_job(
+  my $req = $client->call_remote_async(
       method     => "my.service.baz",
       params     => { %args },
       on_success => sub {
@@ -637,7 +653,7 @@ Version 0.03
       },
   );
   
-  $client->wait_all_jobs;
+  $client->wait_async_calls;
 
 =head1 DESCRIPTION
 
@@ -648,9 +664,9 @@ There are four different methods to do so:
   ┌───────────────────┬──────────────┬────────┬────────┬────────┐
   │ method            │ sent to      │ queued │ result │ blocks │
   ├───────────────────┼──────────────┼────────┼────────┼────────┤
-  │ do_job            │ 1 worker     │ yes    │ yes    │ yes    │
-  │ do_async_job      │ 1 worker     │ yes    │ yes    │ no     │
-  │ do_background_job │ 1 worker     │ yes    │ no     │ no     │
+  │ call_remote       │ 1 worker     │ yes    │ yes    │ yes    │
+  │ call_remote_async │ 1 worker     │ yes    │ yes    │ no     │
+  │ fire_remote       │ 1 worker     │ yes    │ no     │ no     │
   │ send_notification │ many workers │ no     │ no     │ no     │
   └───────────────────┴──────────────┴────────┴────────┴────────┘
 
@@ -717,7 +733,7 @@ Make this client stop accepting specified notifications from message bus.
 
 C<$method> must be one of the strings used previously in C<accept_notifications>.
 
-=head3 do_job ( %args )
+=head3 call_remote ( %args )
 
 Makes a synchronous RPC call to a service worker through the message bus.
 
@@ -758,7 +774,7 @@ an error response. If set to false returns a C<Beekeeper::JSONRPC::Error> instea
 
 =back
 
-=head3 do_async_job ( %args )
+=head3 call_remote_async ( %args )
 
 Makes an asynchronous RPC call to a service worker through the message bus.
 
@@ -766,7 +782,7 @@ It returns immediately a C<Beekeeper::JSONRPC::Request> object which, once compl
 will have a defined C<response>.
 
 This method  accepts parameters C<method>, C<params>, C<address> and C<timeout> 
-the same as C<do_job>. Additionally two callbacks can be specified:
+the same as C<call_remote>. Additionally two callbacks can be specified:
 
 =over 4
 
@@ -782,7 +798,7 @@ C<Beekeeper::JSONRPC::Error> object as parameter. Must be a coderef.
 
 =back
 
-=head3 do_background_job ( %args )
+=head3 fire_remote ( %args )
 
 Makes an asynchronous RPC call to a service worker through the message bus but
 does not expect to receive any response, it is a fire and forget call.
@@ -790,24 +806,23 @@ does not expect to receive any response, it is a fire and forget call.
 It returns undef immediately.
 
 This method  accepts parameters C<method>, C<params>, C<address> and C<timeout> 
-the same as C<do_job>.
+the same as C<call_remote>.
 
-=head3 wait_all_jobs
+=head3 wait_async_calls
 
-Wait (in the event loop) until all calls made by C<do_async_job> are completed.
+Wait (in the event loop) until all calls made by C<call_remote_async> are completed.
 
-=head3 set_auth_tokens ( @tokens )
+=head3 set_authentication_data ( $data )
 
-Add arbitrary auth tokens to subsequent jobs requests or notifications sent.
+Add an arbitrary authentication data blob to subsequent jobs requests or 
+notifications sent.
 
-Workers get the caller tokens already set when executing jobs or notifications 
-callbacks, and then these are piggybacked automatically.
+The meaning of this data is application specific, this framework doesn't give 
+any special one to it.
 
-This framework doesn't give any special meaning to these tokens.
+=head3 get_authentication_data
 
-=head3 get_auth_tokens
-
-Get the list of current auth tokens in use.
+Gets the current authentication data blob.
 
 =head1 SEE ALSO
  
