@@ -16,19 +16,18 @@ use constant CHECK_PERIOD => Beekeeper::Worker::REPORT_STATUS_PERIOD;
 
 sub authorize_request {
     my ($self, $req) = @_;
-    my $required;
 
     if ($req->{method} eq '_bkpr.supervisor.worker_status' ||
         $req->{method} eq '_bkpr.supervisor.worker_exit' ) {
-        $required = 'BKPR_SYSTEM';
+
+        return unless $self->__has_authorization_token('BKPR_SYSTEM');
     }
     else {
-        $required = 'BKPR_ADMIN';
+
+        return unless $self->__has_authorization_token('BKPR_ADMIN');
     }
 
-    return unless $req->has_auth_tokens( $required );
-
-    return REQUEST_AUTHORIZED;
+    return BKPR_REQUEST_AUTHORIZED;
 }
 
 sub on_startup {
@@ -46,7 +45,7 @@ sub on_startup {
         '_bkpr.supervisor.restart_workers' => 'restart_workers',
     );
 
-    $self->accept_jobs(
+    $self->accept_remote_calls(
         '_bkpr.supervisor.worker_status'       => 'worker_status',
         '_bkpr.supervisor.worker_exit'         => 'worker_exit',
         '_bkpr.supervisor.get_workers_status'  => 'get_workers_status',
@@ -242,9 +241,10 @@ sub check_queues {
     # Tell Sinkhole to respond immediately to all requests sent to 
     # unserviced queues with a "Method not available" error response
 
+    my $guard = $self->__use_authorization_token('BKPR_SYSTEM');
+
     $self->send_notification(
         method => '_bkpr.sinkhole.unserviced_queues',
-        __auth => 'BKPR_SYSTEM',
         params => { queues => \@unserviced },
     );
 }
