@@ -57,14 +57,19 @@ sub on_startup    { }
 sub on_shutdown {
     my $self = shift;
 
+    log_info "Shutting down";
+
     # Wait for clients to gracefully disconnect
-    for (1..100) {
-        last unless (keys %{$self->{connections}} <= 1); # our one
+    for (1..60) {
+        my $conn_count = scalar keys %{$self->{connections}};
+        last if $conn_count <= 1; # our one
         my $wait = AnyEvent->condvar;
-        my $tmr = AnyEvent->timer( after => 0.1, cb => $wait );
+        my $tmr = AnyEvent->timer( after => 0.5, cb => $wait );
         $wait->recv;
     }
 
+    undef $self->{_LOGGER}->{_BUS};
+
     # Get rid of our connection to ourselves
     $self->{_BUS}->disconnect;
 }
@@ -72,34 +77,48 @@ sub on_shutdown {
 sub authorize_request {
     my ($self, $req) = @_;
 
-    return REQUEST_AUTHORIZED;
+    return BKPR_REQUEST_AUTHORIZED;
 }
 
 sub start_broker {
-    my $self = shift;
-    weaken($self);
-
-    my $config = Beekeeper::Config->read_config_file( 'toybroker.config.json' ) || {};
-    $self->{config} = $config;
+    my ($self) = @_;
 
     $self->{connections} = {};
     $self->{clients}     = {};
     $self->{topics}      = {};
+    $self->{users}       = {};
 
-    my $max_packet_size = $config->{'max_packet_size'} || 65536;
+    my $config = Beekeeper::Config->read_config_file( 'toybroker.config.json' );
 
-    my $listen_addr = $config->{'listen_addr'} || '127.0.0.1';  # Must be an IPv4 or IPv6 address
-    my $listen_port = $config->{'listen_port'} ||  1883;
+    # Start a default listener if no config found
+    $config = [ {} ] unless defined $config;
 
-    ($listen_addr) = ($listen_addr =~ m/^([\w\.:]+)$/);  # untaint
-    ($listen_port) = ($listen_port =~ m/^(\d+)$/);
+    foreach my $listener (@$config) {
 
-    $self->{listener} = tcp_server ($listen_addr, $listen_port, sub {
-        my ($fh, $host, $port) = @_;
+        if ($listener->{users}) {
+            %{$self->{users}} = ( %{$self->{users}}, %{$listener->{users}} );
+        }
 
-        my $login_tmr = AnyEvent->timer( after => 5, cb => sub {
-            $self->_shutdown($fh) unless $self->get_client($fh);
-        });
+        $self->start_listener( $listener );
+    }
+}
+
+sub start_listener {
+    my ($self, $listener) = @_;
+    weaken($self);
+
+    my $max_packet_size = $listener->{'max_packet_size'} || 65536;
+
+    my $addr = $listener->{'listen_addr'} || '127.0.0.1';  # Must be an IPv4 or IPv6 address
+    my $port = $listener->{'listen_port'} ||  1883;
+
+    ($addr) = ($addr =~ m/^([\w\.:]+)$/);  # untaint
+    ($port) = ($port =~ m/^(\d+)$/);
+
+    log_info "Listening on $addr:$port";
+
+    $self->{"listener-$addr-$port"} = tcp_server ($addr, $port, sub {
+        my ($FH, $host, $port) = @_;
 
         my $packet_type;
         my $packet_flags;
@@ -111,12 +130,11 @@ sub start_broker {
         my $offs;
         my $byte;
 
-        $self->{connections}->{"$fh"} = AnyEvent::Handle->new(
-            fh => $fh,
+        my $fh; $fh = AnyEvent::Handle->new(
+            fh => $FH,
             keepalive => 1,
             no_delay => 1,
             on_read => sub {
-                my $fh = $_[0];
 
                 PARSE_PACKET: {
 
@@ -224,17 +242,22 @@ sub start_broker {
             },
             on_eof => sub {
                 # Clean disconnection, client will not write anymore
-                my $fh = $_[0];
                 $self->remove_client($fh);
                 delete $self->{connections}->{"$fh"};
             },
             on_error => sub {
-                my $fh = $_[0];
                 log_error "$_[2]\n";
                 $self->remove_client($fh);
                 delete $self->{connections}->{"$fh"};
             }
         );
+
+        $self->{connections}->{"$fh"} = $fh;
+
+        #TODO: Close connection on login timeout
+        # my $login_tmr = AnyEvent->timer( after => 5, cb => sub {
+        #     $self->_shutdown($fh) unless $self->get_client($fh);
+        # });
     });
 }
 
@@ -1088,7 +1111,7 @@ sub add_client {
     my $username  = $prop->{'username'};
     my $password  = $prop->{'password'};
 
-    my $users_cfg = $self->{config}->{'users'};
+    my $users_cfg = $self->{'users'};
     my $authorized;
 
     AUTH: {
