@@ -12,6 +12,7 @@ use Beekeeper::JSONRPC;
 use JSON::XS;
 use Time::HiRes;
 use Sys::Hostname;
+use Digest::SHA 'sha256_hex';
 use Scalar::Util 'blessed';
 use Carp;
 
@@ -20,11 +21,11 @@ use Carp;
 use constant COMPILE_ERROR_EXIT_CODE => 99;
 use constant REPORT_STATUS_PERIOD    => 5;
 use constant UNSUBSCRIBE_LINGER      => 2;
-use constant REQUEST_AUTHORIZED      => int(rand(90000000)+10000000);
+use constant BKPR_REQUEST_AUTHORIZED => int(rand(90000000)+10000000);
 
 use Exporter 'import';
 
-our @EXPORT = qw( REQUEST_AUTHORIZED );
+our @EXPORT = qw( BKPR_REQUEST_AUTHORIZED );
 
 our @EXPORT_OK = qw(
     log_fatal
@@ -42,7 +43,7 @@ our @EXPORT_OK = qw(
 our %EXPORT_TAGS = ('log' => [ @EXPORT_OK, @EXPORT ]);
 
 our $Logger = sub { warn(@_) }; # redefined later by __init_logger
-our $LogLevel = LOG_WARN;
+our $LogLevel = LOG_INFO;
 
 sub log_fatal    (@) { $LogLevel >= LOG_FATAL  && $Logger->( LOG_FATAL,  @_ ) }
 sub log_alert    (@) { $LogLevel >= LOG_ALERT  && $Logger->( LOG_ALERT,  @_ ) }
@@ -55,7 +56,8 @@ sub log_info     (@) { $LogLevel >= LOG_INFO   && $Logger->( LOG_INFO,   @_ ) }
 sub log_debug    (@) { $LogLevel >= LOG_DEBUG  && $Logger->( LOG_DEBUG,  @_ ) }
 sub log_trace    (@) { $LogLevel >= LOG_TRACE  && $Logger->( LOG_TRACE,  @_ ) }
 
-our $JSON;
+my %AUTH_TOKENS;
+my $JSON;
 
 
 sub new {
@@ -120,6 +122,8 @@ sub new {
         # Pass broker connection to logger
         $self->{_LOGGER}->{_BUS} = $self->{_BUS} if (exists $self->{_LOGGER}->{_BUS});
 
+        $self->__init_auth_tokens;
+
         $self->__init_worker;
     };
 
@@ -132,6 +136,28 @@ sub new {
     return $self;
 }
 
+sub __init_auth_tokens {
+    my ($self) = @_;
+
+    my $secret = 'salt'; #TODO: read from config file
+
+    $AUTH_TOKENS{'BKPR_SYSTEM'} = sha256_hex('BKPR_SYSTEM'. $secret);
+    $AUTH_TOKENS{'BKPR_ADMIN'}  = sha256_hex('BKPR_ADMIN' . $secret);
+    $AUTH_TOKENS{'BKPR_ROUTER'} = sha256_hex('BKPR_ROUTER'. $secret);
+}
+
+sub __has_authorization_token {
+    my ($self, $auth_level) = @_;
+
+    my $auth_data = $self->{_CLIENT}->{auth_data};
+
+    return 0 unless $auth_data && $auth_level;
+    return 0 unless exists $AUTH_TOKENS{$auth_level};
+    return 0 unless $AUTH_TOKENS{$auth_level} eq $auth_data;
+
+    return 1;
+}
+
 sub __init_logger {
     my $self = shift;
 
@@ -238,7 +264,7 @@ sub accept_notifications {
         croak "Already accepting notifications $fq_meth" if exists $callbacks->{"msg.$fq_meth"};
         $callbacks->{"msg.$fq_meth"} = $callback;
 
-        my $local_bus = $self->{_BUS}->{cluster};
+        my $local_bus = $self->{_BUS}->{bus_role};
 
         my $topic = "msg/$local_bus/$service/$method";
         $topic =~ tr|.*|/#|;
@@ -284,7 +310,7 @@ sub __get_cb_coderef {
 }
 
 
-sub accept_jobs {
+sub accept_remote_calls {
     my ($self, %args) = @_;
 
     my $worker = $self->{_WORKER};
@@ -311,7 +337,7 @@ sub accept_jobs {
             carp "Running multiple services within a single worker hurts load balancing (don't do that)";
         }
 
-        my $local_bus = $self->{_BUS}->{cluster};
+        my $local_bus = $self->{_BUS}->{bus_role};
 
         my $queue = "\$share/BKPR/req/$local_bus/$service";
         $queue =~ tr|.*|/#|;
@@ -381,7 +407,7 @@ sub __drain_task_queue {
                 }
 
                 bless $request, 'Beekeeper::JSONRPC::Notification';
-                $request->{_headers} = $msg_headers;
+                $request->{_mqtt_prop} = $msg_headers;
 
                 my $method = $request->{method};
 
@@ -393,7 +419,11 @@ sub __drain_task_queue {
                 my $cb = $worker->{callbacks}->{"msg.$1.$2"} || 
                          $worker->{callbacks}->{"msg.$1.*"};
 
-                unless (($self->authorize_request($request) || "") eq REQUEST_AUTHORIZED) {
+                local $client->{caller_id}   = $msg_headers->{'clid'};
+                local $client->{caller_addr} = $msg_headers->{'addr'};
+                local $client->{auth_data}   = $msg_headers->{'auth'};
+
+                unless (($self->authorize_request($request) || "") eq BKPR_REQUEST_AUTHORIZED) {
                     log_warn "Notification $method was not authorized";
                     return;
                 }
@@ -403,10 +433,6 @@ sub __drain_task_queue {
                     return;
                 }
 
-                local $client->{curr_request} = $request;
-                local $client->{auth_tokens}  = $msg_headers->{'x-auth-tokens'};
-                local $client->{session_id}   = $msg_headers->{'x-session'};
-
                 $cb->($self, $request->{params}, $request);
             };
 
@@ -438,7 +464,7 @@ sub __drain_task_queue {
                 my $method  = $request->{method};
 
                 bless $request, 'Beekeeper::JSONRPC::Request';
-                $request->{_headers} = $msg_headers;
+                $request->{_mqtt_prop} = $msg_headers;
 
                 unless (defined $method && $method =~ m/^([\.\w-]+)\.([\w-]+)$/) {
                     log_warn "Received request with invalid method $method";
@@ -448,7 +474,11 @@ sub __drain_task_queue {
                 my $cb = $worker->{callbacks}->{"req.$1.$2"} || 
                          $worker->{callbacks}->{"req.$1.*"};
 
-                unless (($self->authorize_request($request) || "") eq REQUEST_AUTHORIZED) {
+                local $client->{caller_id}   = $msg_headers->{'clid'};
+                local $client->{caller_addr} = $msg_headers->{'addr'};
+                local $client->{auth_data}   = $msg_headers->{'auth'};
+
+                unless (($self->authorize_request($request) || "") eq BKPR_REQUEST_AUTHORIZED) {
                     log_warn "Request $method was not authorized";
                     die Beekeeper::JSONRPC::Error->request_not_authorized;
                 }
@@ -458,10 +488,6 @@ sub __drain_task_queue {
                     die Beekeeper::JSONRPC::Error->method_not_found;
                 }
 
-                local $client->{curr_request} = $request;
-                local $client->{auth_tokens}  = $msg_headers->{'x-auth-tokens'};
-                local $client->{session_id}   = $msg_headers->{'x-session'};
-
                 # Execute job
                 $cb->($self, $request->{params}, $request);
             };
@@ -515,7 +541,7 @@ sub __drain_task_queue {
 
                 $self->{_BUS}->publish(
                     topic     => $msg_headers->{'response_topic'},
-                    fwd_reply => $msg_headers->{'fwd_reply'},
+                    addr      => $msg_headers->{'addr'},
                     payload   => \$json,
                     buffer_id => 'response',
                 );
@@ -582,7 +608,7 @@ sub stop_accepting_notifications {
             next;
         }
 
-        my $local_bus = $self->{_BUS}->{cluster};
+        my $local_bus = $self->{_BUS}->{bus_role};
 
         my $topic = "msg/$local_bus/$service/$method";
         $topic =~ tr|.*|/#|;
@@ -620,7 +646,7 @@ sub stop_accepting_notifications {
 }
 
 
-sub stop_accepting_jobs {
+sub stop_accepting_calls {
     my ($self, @methods) = @_;
 
     croak "No method specified" unless @methods;
@@ -651,7 +677,7 @@ sub stop_accepting_jobs {
             next;
         }
 
-        my $local_bus = $self->{_BUS}->{cluster};
+        my $local_bus = $self->{_BUS}->{bus_role};
 
         my $topic = "\$share/BKPR/req/$local_bus/$service";
         $topic =~ tr|.*|/#|;
@@ -680,7 +706,7 @@ sub stop_accepting_jobs {
             on_unsuback  => sub {
                 my ($success, $prop) = @_;
 
-                #TODO: Report caller of stop_accepting_jobs method
+                #TODO: Report caller of stop_accepting_calls method
                 warn "Could not unsubscribe from $topic" unless $success; 
 
                 my $postponed = $worker->{postponed} ||= [];
@@ -754,7 +780,7 @@ sub stop_working {
 
         $worker->{stop_cv}->begin;
 
-        $self->stop_accepting_jobs( $service . '.*' );
+        $self->stop_accepting_calls( $service . '.*' );
     }
 }
 
@@ -763,6 +789,7 @@ sub __report_status {
     my $self = shift;
 
     my $worker = $self->{_WORKER};
+    my $client = $self->{_CLIENT};
 
     my $now = Time::HiRes::time;
     my $period = $now - ($worker->{last_report} || ($now - 1));
@@ -790,10 +817,12 @@ sub __report_status {
         $queues{$1} = 1;
     }
 
+    local $client->{auth_data} = $AUTH_TOKENS{'BKPR_SYSTEM'};
+    local $client->{caller_id};
+
     # Tell any supervisor our stats
-    $self->do_background_job(
+    $self->fire_remote(
         method => '_bkpr.supervisor.worker_status',
-        __auth => 'BKPR_SYSTEM',
         params => {
             class => ref($self),
             host  => $worker->{hostname},
@@ -813,10 +842,13 @@ sub __report_exit {
     return unless $self->{_BUS}->{is_connected};
 
     my $worker = $self->{_WORKER};
+    my $client = $self->{_CLIENT};
 
-    $self->do_background_job(
+    local $client->{auth_data} = $AUTH_TOKENS{'BKPR_SYSTEM'};
+    local $client->{caller_id};
+
+    $self->fire_remote(
         method => '_bkpr.supervisor.worker_exit',
-        __auth => 'BKPR_SYSTEM',
         params => {
             class => ref($self),
             host  => $worker->{hostname},
@@ -856,7 +888,7 @@ Version 0.03
           'myapp.msg' => 'got_message',
       );
       
-      $self->accept_jobs(
+      $self->accept_remote_calls(
           'myapp.sum' => 'do_sum',
       );
   
@@ -866,7 +898,7 @@ Version 0.03
   sub authorize_request {
       my ($self, $req) = @_;
   
-      return REQUEST_AUTHORIZED;
+      return BKPR_REQUEST_AUTHORIZED;
   }
   
   sub got_message {
@@ -916,7 +948,7 @@ This method MUST be overrided in your worker classes, as the default behavior is
 to deny the execution of any request.
 
 When a request is received this method is called before executing the corresponding
-callback, and it must return the exported constant C<REQUEST_AUTHORIZED> in order to
+callback, and it must return the exported constant C<BKPR_REQUEST_AUTHORIZED> in order to
 authorize it. Returning any other value will result in the request being ignored. 
 
 This is the place to handle application authentication and authorization.
@@ -948,7 +980,7 @@ logged, unless you specifically catch and ignore them.
 =head3 Client methods
 
 In order to make RPC calls to another services, methods C<send_notification>, 
-C<do_job>, C<do_async_job>, C<do_background_job> and C<wait_all_jobs> are 
+C<call_remote>, C<call_remote_async>, C<fire_remote> and C<wait_async_calls> are 
 automatically imported from C<Beekeeper::Client>.
 
 =head3 accept_notifications ( $method => $callback, ... )
@@ -993,7 +1025,7 @@ Example:
        # $req is a Beekeeper::JSONRPC::Notification object
   }
 
-=head3 accept_jobs ( $method => $callback, ... )
+=head3 accept_remote_calls ( $method => $callback, ... )
 
 Make this worker start accepting specified RPC requests from message bus.
 
@@ -1021,7 +1053,7 @@ Example:
   sub on_startup {
       my $self = shift;
       
-      $self->accept_jobs(
+      $self->accept_remote_calls(
           'foo.inc' => 'inc',       # call $self->inc for requests to 'foo.inc'
           'foo.baz' => $coderef,    # call $self->$coderef for requests to 'foo.baz'
           'foo.*'   => 'fallback',  # call $self->fallback for any other 'foo.*'
@@ -1044,11 +1076,11 @@ Make this worker stop accepting specified notifications from message bus.
 
 C<$method> must be one of the strings used previously in C<accept_notifications>.
 
-=head3 stop_accepting_jobs ( $method, ... )
+=head3 stop_accepting_calls ( $method, ... )
 
 Make this worker stop accepting specified RPC requests from message bus.
 
-C<$method> must be one of the strings used previously in C<accept_jobs>.
+C<$method> must be one of the strings used previously in C<accept_remote_calls>.
 
 =head3 stop_working 
 
