@@ -3,29 +3,7 @@ package Beekeeper::Worker::Util;
 use strict;
 use warnings;
 
-our $VERSION = '0.01';
-
-=head1 NAME
- 
-Beekeeper::Worker::Util
- 
-=head1 VERSION
- 
-Version 0.01
-
-=head1 SYNOPSIS
-
-  use Beekeeper::Worker::Util 'shared_cache';
-  
-  my $c = $self->shared_cache( ... );
-  
-  $c->set( $key => $value );
-  $c->get( $key );
-  $c->delete( $key );
-
-=head1 DESCRIPTION
-
-=cut
+our $VERSION = '0.03';
 
 use Beekeeper::Worker::Util::SharedCache;
 
@@ -44,15 +22,41 @@ sub shared_cache {
 
 1;
 
+__END__
+
+=pod
+
 =encoding utf8
 
+=head1 NAME
+ 
+Beekeeper::Worker::Util
+ 
+=head1 VERSION
+ 
+Version 0.03
+
+=head1 SYNOPSIS
+
+  use Beekeeper::Worker::Util 'shared_cache';
+  
+  my $c = $self->shared_cache( ... );
+  
+  $c->set( $key => $value );
+  $c->get( $key );
+  $c->delete( $key );
+
+=head1 SEE ALSO
+ 
+L<Beekeeper::Worker::Util::SharedCache>
+
 =head1 AUTHOR
 
 José Micó, C<jose.mico@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2015 José Micó.
+Copyright 2015-2021 José Micó.
 
 This is free software; you can redistribute it and/or modify it under the same 
 terms as the Perl 5 programming language itself.
