@@ -11,8 +11,9 @@ my $bus_config_json = qq<
 
 [
     {
-        "bus-id"   : "test",
+        "bus_id"   : "test",
         "host"     : "localhost",
+        "port"     :  %PORT%,
         "username" : "test",
         "password" : "abc123",
     },
@@ -22,14 +23,16 @@ my $toybroker_config_json = qq<
 
 # Mock toybroker.config.json file used by tests
 
-{
-    "listen_addr" : "127.0.0.1",
-    "listen_port" : "1883",
+[
+    {
+        "listen_addr" : "127.0.0.1",
+        "listen_port" : %PORT%,
 
-    "users" : {
-        "test" : { "password" : "abc123" },
+        "users" : {
+            "test" : { "password" : "abc123" },
+        },
     },
-}>;
+]>;
 
 sub read_config_file {
     my ($class, $file) = @_;
@@ -37,6 +40,10 @@ sub read_config_file {
     my $data = $file eq "bus.config.json"       ? $bus_config_json       : 
                $file eq "toybroker.config.json" ? $toybroker_config_json : '';
 
+    #TODO: Find an unused port
+    my $test_port = 51883;
+    $data =~ s/%PORT%/$test_port/;
+
     # Allow comments and end-comma
     my $json = JSON::XS->new->relaxed;
 
